@@ -37,3 +37,26 @@ class AristaEos(netmiko_devices.NetmikoSwitch):
         'no switchport mode trunk',
         'switchport trunk allowed vlan none'
     )
+
+    SET_NATIVE_VLAN = (
+        'interface {port}',
+        'switchport mode trunk',
+        'switchport trunk native vlan {segmentation_id}',
+        'switchport trunk allowed vlan add {segmentation_id}'
+    )
+
+    DELETE_NATIVE_VLAN = (
+        'interface {port}',
+        'no switchport trunk native vlan {segmentation_id}',
+        'switchport trunk allowed vlan remove {segmentation_id}',
+    )
+
+    ADD_NETWORK_TO_TRUNK = (
+        'interface {port}',
+        'switchport trunk allowed vlan add {segmentation_id}'
+    )
+
+    REMOVE_NETWORK_FROM_TRUNK = (
+        'interface {port}',
+        'switchport trunk allowed vlan remove {segmentation_id}'
+    )
