@@ -18,12 +18,16 @@ from neutron.db import provisioning_blocks
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as const
+from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
 from oslo_log import log as logging
 
 from networking_generic_switch import config as gsw_conf
 from networking_generic_switch import devices
 from networking_generic_switch.devices import utils as device_utils
+from networking_generic_switch import exceptions as ngs_exc
+from networking_generic_switch import trunk_driver
+from networking_generic_switch import utils as ngs_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -32,6 +36,10 @@ GENERIC_SWITCH_ENTITY = 'GENERICSWITCH'
 
 class GenericSwitchDriver(api.MechanismDriver):
 
+    @property
+    def connectivity(self):
+        return portbindings.CONNECTIVITY_L2
+
     def initialize(self):
         """Perform driver initialization.
 
@@ -53,6 +61,8 @@ class GenericSwitchDriver(api.MechanismDriver):
         if not self.switches:
             LOG.error('No devices have been loaded')
 
+        self.trunk_driver = trunk_driver.GenericSwitchTrunkDriver.create(self)
+
     def create_network_precommit(self, context):
         """Allocate resources for a new network.
 
@@ -341,7 +351,7 @@ class GenericSwitchDriver(api.MechanismDriver):
         state changes that it does not know or care about.
         """
         port = context.current
-        network = context.network.current
+        segment = context.top_bound_segment
         if self._is_port_bound(port):
             binding_profile = port['binding:profile']
             local_link_information = binding_profile.get(
@@ -358,9 +368,9 @@ class GenericSwitchDriver(api.MechanismDriver):
                 return
             # If binding has already succeeded, we should have valid links
             # at this point, but check just in case.
-            if not self._is_link_valid(port, network):
+            if not self._is_link_valid(port, segment):
                 return
-            is_802_3ad = self._is_802_3ad(port)
+            is_802_3ad = ngs_utils.is_802_3ad(binding_profile)
             for link in local_link_information:
                 port_id = link.get('port_id')
                 switch_info = link.get('switch_info')
@@ -370,16 +380,32 @@ class GenericSwitchDriver(api.MechanismDriver):
                     ngs_mac_address=switch_id)
 
                 # If segmentation ID is None, set vlan 1
-                segmentation_id = network.get('provider:segmentation_id') or 1
+                segmentation_id = segment.get(api.SEGMENTATION_ID) or 1
                 LOG.debug("Putting switch port %(switch_port)s on "
                           "%(switch_info)s in vlan %(segmentation_id)s",
                           {'switch_port': port_id, 'switch_info': switch_info,
                            'segmentation_id': segmentation_id})
+                trunk_details = port.get('trunk_details', {})
+                plug_kwargs = {}
+                if trunk_details:
+                    plug_kwargs["trunk_details"] = trunk_details
                 # Move port to network
-                if is_802_3ad and hasattr(switch, 'plug_bond_to_network'):
-                    switch.plug_bond_to_network(port_id, segmentation_id)
+                if is_802_3ad:
+                    if (trunk_details and not
+                            switch.support_trunk_on_bond_ports):
+                        raise ngs_exc.GenericSwitchNotSupported(
+                            "Trunks are not supported by "
+                            "networking-generic-switch %s.",
+                            switch.device_name)
+                    switch.plug_bond_to_network(port_id, segmentation_id,
+                                                **plug_kwargs)
                 else:
-                    switch.plug_port_to_network(port_id, segmentation_id)
+                    if trunk_details and not switch.support_trunk_on_ports:
+                        raise ngs_exc.GenericSwitchNotSupported(
+                            feature="trunks",
+                            switch=switch.device_name)
+                    switch.plug_port_to_network(port_id, segmentation_id,
+                                                **plug_kwargs)
                 LOG.info("Successfully plugged port %(port_id)s in segment "
                          "%(segment_id)s on device %(device)s",
                          {'port_id': port['id'], 'device': switch_info,
@@ -388,12 +414,19 @@ class GenericSwitchDriver(api.MechanismDriver):
             provisioning_blocks.provisioning_complete(
                 context._plugin_context, port['id'], resources.PORT,
                 GENERIC_SWITCH_ENTITY)
+            for subport in port.get('trunk_details', {}).get("sub_ports", []):
+                subport_obj = context._plugin.get_port(context.plugin_context,
+                                                       subport['port_id'])
+                if subport_obj['status'] != const.PORT_STATUS_ACTIVE:
+                    context._plugin.update_port_status(
+                        context.plugin_context, subport["port_id"],
+                        const.PORT_STATUS_ACTIVE)
         elif self._is_port_bound(context.original):
             # The port has been unbound. This will cause the local link
-            # information to be lost, so remove the port from the network on
+            # information to be lost, so remove the port from the segment on
             # the switch now while we have the required information.
-            self._unplug_port_from_network(context.original,
-                                           context.network.current)
+            self._unplug_port_from_segment(context.original,
+                                           context.original_top_bound_segment)
 
     def delete_port_precommit(self, context):
         """Delete resources of a port.
@@ -422,7 +455,7 @@ class GenericSwitchDriver(api.MechanismDriver):
 
         port = context.current
         if self._is_port_bound(port):
-            self._unplug_port_from_network(port, context.network.current)
+            self._unplug_port_from_segment(port, context.top_bound_segment)
 
     def bind_port(self, context):
         """Attempt to bind a port.
@@ -466,55 +499,38 @@ class GenericSwitchDriver(api.MechanismDriver):
         can use with ports.
         """
 
+        # NOTE(vsaienko): Important that this method remains lightweight.
+        # The actual port handling is done in update_port_postcommit.
+        # For more info please read docstring.
+
         port = context.current
-        network = context.network.current
         binding_profile = port['binding:profile']
         local_link_information = binding_profile.get('local_link_information')
 
         if self._is_port_supported(port) and local_link_information:
+            # Filter segments where port is already assigned to subnet(s)
+            subnets = []
+            for fixed_ip in port.get('fixed_ips', []):
+                subnet_id = fixed_ip.get('subnet_id')
+                if subnet_id:
+                    subnets.append(context._plugin.get_subnet(
+                        context.plugin_context,
+                        subnet_id))
+            segments = []
+            if len(subnets) > 0:
+                for segment in context.segments_to_bind:
+                    for subnet in subnets:
+                        segment_id = subnet.get('segment_id')
+                        if segment_id is None or segment_id == segment[api.ID]:
+                            segments.append(segment)
+            else:
+                segments = context.segments_to_bind
+
             # NOTE(jamesdenton): If any link of the port is invalid, none
             # of the links should be processed.
-            if not self._is_link_valid(port, network):
+            if not self._is_link_valid(port, segments[0]):
                 return
 
-            is_802_3ad = self._is_802_3ad(port)
-            for link in local_link_information:
-                port_id = link.get('port_id')
-                switch_info = link.get('switch_info')
-                switch_id = link.get('switch_id')
-                switch = device_utils.get_switch_device(
-                    self.switches, switch_info=switch_info,
-                    ngs_mac_address=switch_id)
-
-                segments = context.segments_to_bind
-                # If segmentation ID is None, set vlan 1
-                segmentation_id = segments[0].get('segmentation_id') or 1
-
-                # Fail if port or vlan not in allow list
-                if not switch.is_allowed(port_id, segmentation_id):
-                    LOG.warn("Skipped binding port %(port_id)s, "
-                             "port %(port)s in segment "
-                             "%(segment_id)s on device %(device)s, as either "
-                             "the port or vlan is not on the allow list",
-                             {'port_id': port['id'], 'port': port_id,
-                              'device': switch_info,
-                              'segment_id': segmentation_id})
-                    return
-
-                LOG.debug("Putting port %(port_id)s on %(switch_info)s "
-                          "to vlan: %(segmentation_id)s",
-                          {'port_id': port_id, 'switch_info': switch_info,
-                           'segmentation_id': segmentation_id})
-                # Move port to network
-                if is_802_3ad and hasattr(switch, 'plug_bond_to_network'):
-                    switch.plug_bond_to_network(port_id, segmentation_id)
-                else:
-                    switch.plug_port_to_network(port_id, segmentation_id)
-                LOG.info("Successfully bound port %(port_id)s in segment "
-                         "%(segment_id)s on device %(device)s",
-                         {'port_id': port['id'], 'device': switch_info,
-                          'segment_id': segmentation_id})
-
             context.set_binding(segments[0][api.ID],
                                 portbindings.VIF_TYPE_OTHER, {})
 
@@ -522,8 +538,8 @@ class GenericSwitchDriver(api.MechanismDriver):
                 context._plugin_context, port['id'], resources.PORT,
                 GENERIC_SWITCH_ENTITY)
 
-    def _is_link_valid(self, port, network):
-        """Return whether a link references valid switch and physnet.
+    def _is_link_valid(self, port, segment):
+        """Return whether a link references valid switch and segment.
 
         If the local link information refers to a switch that is not
         known to NGS or the switch is not associated with the respective
@@ -531,7 +547,7 @@ class GenericSwitchDriver(api.MechanismDriver):
         be raised.
 
         :param port: The port to check
-        :param network: the network mapped to physnet
+        :param segment: The segment to check against
         :returns: Whether the link refers to a configured switch and/or switch
                   is associated with physnet
         """
@@ -540,6 +556,7 @@ class GenericSwitchDriver(api.MechanismDriver):
         local_link_information = binding_profile.get('local_link_information')
 
         for link in local_link_information:
+            port_id = link.get('port_id')
             switch_info = link.get('switch_info')
             switch_id = link.get('switch_id')
             switch = device_utils.get_switch_device(
@@ -553,8 +570,9 @@ class GenericSwitchDriver(api.MechanismDriver):
                            'device': switch_info})
                 return False
 
-            physnet = network['provider:physical_network']
+            physnet = segment.get(api.PHYSICAL_NETWORK)
             switch_physnets = switch._get_physical_networks()
+            segmentation_id = segment.get(api.SEGMENTATION_ID) or 1
 
             if switch_physnets and physnet not in switch_physnets:
                 LOG.error("Cannot bind port %(port)s as device %(device)s "
@@ -563,6 +581,17 @@ class GenericSwitchDriver(api.MechanismDriver):
                           {'port': port['id'], 'device': switch_info,
                            'physnet': physnet})
                 return False
+
+            # Fail if port or vlan not in allow list
+            if not switch.is_allowed(port_id, segmentation_id):
+                LOG.warn("Skipped binding port %(port_id)s, "
+                         "port %(port)s in segment "
+                         "%(segment_id)s on device %(device)s, as either "
+                         "the port or vlan is not on the allow list",
+                         {'port_id': port['id'], 'port': port_id,
+                          'device': switch_info,
+                          'segment_id': segmentation_id})
+                return False
         return True
 
     @staticmethod
@@ -592,37 +621,22 @@ class GenericSwitchDriver(api.MechanismDriver):
         vif_type = port[portbindings.VIF_TYPE]
         return vif_type == portbindings.VIF_TYPE_OTHER
 
-    @staticmethod
-    def _is_802_3ad(port):
-        """Return whether a port is using 802.3ad link aggregation.
-
-        :param port: The port to check
-        :returns: Whether the port is a port group using 802.3ad link
-                  aggregation.
-        """
-        binding_profile = port['binding:profile']
-        local_group_information = binding_profile.get(
-            'local_group_information')
-        if not local_group_information:
-            return False
-        return local_group_information.get('bond_mode') in ['4', '802.3ad']
-
-    def _unplug_port_from_network(self, port, network):
-        """Unplug a port from a network.
+    def _unplug_port_from_segment(self, port, segment):
+        """Unplug a port from a segment.
 
         If the configuration required to unplug the port is not present
         (e.g. local link information), the port will not be unplugged and no
         exception will be raised.
 
         :param port: The port to unplug
-        :param network: The network from which to unplug the port
+        :param segment: The segment from which to unplug the port
         """
         binding_profile = port['binding:profile']
         local_link_information = binding_profile.get('local_link_information')
         if not local_link_information:
             return
 
-        is_802_3ad = self._is_802_3ad(port)
+        is_802_3ad = ngs_utils.is_802_3ad(binding_profile)
         for link in local_link_information:
             switch_info = link.get('switch_info')
             switch_id = link.get('switch_id')
@@ -633,13 +647,13 @@ class GenericSwitchDriver(api.MechanismDriver):
                 continue
             port_id = link.get('port_id')
             # If segmentation ID is None, set vlan 1
-            segmentation_id = network.get('provider:segmentation_id') or 1
+            segmentation_id = segment.get(api.SEGMENTATION_ID) or 1
             LOG.debug("Unplugging port %(port)s on %(switch_info)s from vlan: "
                       "%(segmentation_id)s",
                       {'port': port_id, 'switch_info': switch_info,
                        'segmentation_id': segmentation_id})
             try:
-                if is_802_3ad and hasattr(switch, 'unplug_bond_from_network'):
+                if is_802_3ad:
                     switch.unplug_bond_from_network(port_id, segmentation_id)
                 else:
                     switch.delete_port(port_id, segmentation_id)
@@ -647,12 +661,13 @@ class GenericSwitchDriver(api.MechanismDriver):
                 LOG.error("Failed to unplug port %(port_id)s "
                           "on device: %(switch)s from network %(net_id)s "
                           "reason: %(exc)s",
-                          {'port_id': port['id'], 'net_id': network['id'],
+                          {'port_id': port['id'],
+                           'net_id': segment['network_id'],
                            'switch': switch_info, 'exc': e})
                 raise e
             LOG.info('Port %(port_id)s has been unplugged from network '
                      '%(net_id)s on device %(device)s',
-                     {'port_id': port['id'], 'net_id': network['id'],
+                     {'port_id': port['id'], 'net_id': segment['network_id'],
                       'device': switch_info})
 
     def _get_devices_by_physnet(self, physnet):
@@ -668,3 +683,82 @@ class GenericSwitchDriver(api.MechanismDriver):
             # follow the old behaviour of mapping all networks to it.
             if not physnets or physnet in physnets:
                 yield switch_name, switch
+
+    def subports_added(self, context, port, subports):
+        """Tell the agent about new subports to add.
+
+        :param context: Request context
+        :param port: Port dictionary
+        :subports: List with subports
+        """
+
+        # set the correct state on port in the case where it has subports.
+        # If the parent port has been deleted then that delete will handle
+        # removing the trunked vlans on the switch using the mac
+        if not port:
+            LOG.debug('Discarding attempt to ensure subports on a port'
+                      'that has been deleted')
+            return
+
+        if not self._is_port_supported(port):
+            return
+
+        binding_profile = port['binding:profile']
+        local_link_information = binding_profile.get('local_link_information')
+
+        if not local_link_information:
+            return
+
+        for link in local_link_information:
+            port_id = link.get('port_id')
+            switch_info = link.get('switch_info')
+            switch_id = link.get('switch_id')
+            switch = device_utils.get_switch_device(
+                self.switches, switch_info=switch_info,
+                ngs_mac_address=switch_id)
+
+            switch.add_subports_on_trunk(
+                binding_profile, port_id, subports)
+
+        core_plugin = directory.get_plugin()
+
+        for subport in subports:
+            subport_obj = core_plugin.get_port(context,
+                                               subport['port_id'])
+            if subport_obj['status'] != const.PORT_STATUS_ACTIVE:
+                core_plugin.update_port_status(
+                    context, subport["port_id"],
+                    const.PORT_STATUS_ACTIVE)
+
+    def subports_deleted(self, context, port, subports):
+        """Tell the agent about subports to delete.
+
+        :param context: Request context
+        :param port: Port dictionary
+        :subports: List with subports
+        """
+
+        if not port:
+            LOG.debug('Discarding attempt to ensure subports on a port'
+                      'that has been deleted')
+            return
+
+        if not self._is_port_supported(port):
+            return
+
+        binding_profile = port['binding:profile']
+        local_link_information = binding_profile.get('local_link_information')
+
+        if not local_link_information:
+            return
+
+        for link in local_link_information:
+            port_id = link.get('port_id')
+            switch_info = link.get('switch_info')
+            switch_id = link.get('switch_id')
+            switch = device_utils.get_switch_device(
+                self.switches, switch_info=switch_info,
+                ngs_mac_address=switch_id)
+
+            switch.del_subports_on_trunk(
+                binding_profile, port_id, subports)
